import { Component, OnInit } from '@angular/core';
import { MediaChange, MediaObserver } from '@angular/flex-layout';
import {
    AbstractControl,
    FormBuilder,
    FormGroup,
    ValidatorFn,
    Validators
} from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';

import { faCheck } from '@fortawesome/free-solid-svg-icons';
import { Subscription } from 'rxjs';

import { MembershipType } from '@account/models/membership.model';
import {
    navigateToLogin,
    ProfileService,
    storeRedirect
} from '@account/http/profile.service';

const noDelay = 0;

export function loginValidator(): ValidatorFn {
    return (loginGroup: FormGroup) => {
        let valid = true;
        const federatedToken = loginGroup.controls['federatedToken'];
        const userEnteredEmail  = loginGroup.controls['userEnteredEmail'];
        const password = loginGroup.controls['password'];

        if (federatedToken.value) {
            if (userEnteredEmail.value || password.value) {
                valid = false;
            }
        } else {
            if (!userEnteredEmail.valid || !password.value) {
                valid = false;
            }
        }
        return valid ? null : {loginInvalid: true};
    };
}

export function membershipValidator(): ValidatorFn {
    return (supportGroup: FormGroup) => {
        let valid = true;
        const membershipType = supportGroup.controls['membershipType'];
        const paymentToken  = supportGroup.controls['paymentToken'];

        if (membershipType.value) {
            if (!paymentToken.value) {
                valid = false;
            }
        }
        return valid ? null : {membershipInvalid: true};
    };
}


@Component({
  selector: 'account-create-account',
  templateUrl: './new.component.html',
  styleUrls: ['./new.component.scss']
})
export class NewComponent implements OnInit {
    public alignVertical: boolean;
    private mediaWatcher: Subscription;
    public membershipControl: AbstractControl;
    public membershipTypes: MembershipType[];
    public newAcctForm: FormGroup;
    public openDatasetControl: AbstractControl;
    public stepDoneIcon = faCheck;
    public usernameControl: AbstractControl;

    constructor(
        private formBuilder: FormBuilder,
        public mediaObserver: MediaObserver,
        private profileService: ProfileService,
        private route: ActivatedRoute,
        private router: Router
    ) {
        this.mediaWatcher = mediaObserver.media$.subscribe(
            (change: MediaChange) => {
                this.alignVertical = ['xs', 'sm'].includes(change.mqAlias);
            }
        );
    }

    ngOnInit() {
        this.route.data.subscribe(
            (data: {membershipTypes: MembershipType[]}) => {
                this.membershipTypes = data.membershipTypes;
            }
        );
        storeRedirect();
        this.buildForm();
        this.setControlFormAliases();
    }

    private buildForm() {
        const membershipGroup = this.formBuilder.group(
            {
                newMembership: [null],
                membershipType: [null],
                paymentMethod: [null],
                paymentToken: [null]
            },
            {validator: membershipValidator()}
        );

        this.newAcctForm = this.formBuilder.group({
            username: ['', Validators.required],
            membership: membershipGroup,
            openDataset: [null, Validators.required],
        });
    }

    private setControlFormAliases() {
        this.usernameControl = this.newAcctForm.controls['username'];
        this.membershipControl = this.newAcctForm.controls['membership'];
        this.openDatasetControl = this.newAcctForm.controls['openDataset'];
    }

    onFormSubmit() {
        const newValues = this.newAcctForm.value;
        if (!newValues.membership.newMembership) {
            delete newValues.membership;
        }
        console.log(newValues);
        this.profileService.updateAccount(newValues).subscribe(
            () => { this.router.navigate(['/']); }
        );
    }
}
